#ifndef PV_TYPES_H
#define PV_TYPES_H

#include <ptxlist.h>

#include <jv_types.h>
#include <libpv.h>

#define MAX_CONNECTIONS		10
#define XML_STR_BUF_SIZE	80

int pv_verbose;
#define VFPRINTF(verbose, args...)  if(pv_verbose >= (verbose)) fprintf(args)

#ifndef MIN
#define	MIN(a,b) (((a)<(b))?(a):(b))
#endif

#ifndef MAX
#define	MAX(a,b) (((a)>(b))?(a):(b))
#endif


struct pv_backing_store;
typedef struct pv_backing_store pv_backing_store_t;


struct pv_backing_store {
	int	(*init)		(pv_backing_store_t	*,
				 const char		*filename);

	int	(*exit	)	(pv_backing_store_t	*);

	int	(*read)		(pv_backing_store_t	*,
				 off_t			,
				 size_t			);

	int	(*write)	(pv_backing_store_t	*,
				 off_t			,
				 size_t			);

	void	*map;		/*
				 * set to NULL for not mmappable backing stores, 
				 * the user will be prompted for a shm segment
				 */
	size_t	size;

	void	*priv;
};


struct pv_engine {
	/* JVisu */
	int	jv_port;			/* TCP port for JVisu; default=2202 */
	struct jvisu_sock
		jv_active_sock[MAX_CONNECTIONS + 1];
	int	jv_connections; 
	fd_set	jv_read_fdset;
	fd_set	jv_write_fdset;
	ptxlist	*jv_varlist; 
	int	jv_registeredvars;		/* FIXME: replace */
	struct timeval
		jv_tact;

	/* more - JVisu */
	pthread_t
		jvsocket_thread;
	int	jvisu_stop;		/* set 1 to stop JVisu server       */

	
	/* XML PV Description */
	char	*xml_filename;		/* XML descr. of process variables  */

	/* PV Backing Store */
	char	*pv_filename;		/* process variable backing store   */
	int	pv_store_size;
	int	pv_sem;			/* SysV semaphore                   */
	int	pv_sem_key;
	int	pv_shmid;
	void	*pv_map;		/* mmap area                        */
	pv_backing_store_t
		store;
	
	/* PV Data */
	ptxlist	*pv_list;		/* list of process variables        */
};


enum pv_type {
	pv_type_undef = 0,
	pv_type_uint8,
	pv_type_uint16,
	pv_type_uint32,
	pv_type_string,
};


enum pv_unit {
	pv_unit_none = 0,
};


enum pv_scale {
	pv_scale_none = 0,
	pv_scale_giga,
	pv_scale_mega,
	pv_scale_kilo,
	pv_scale_milli,
	pv_scale_micro,
	pv_scale_nano,
	pv_scale_pico,
	pv_scale_custom,
};


typedef struct {
	char 		*id;
	enum pv_type 	type;
	unsigned long 	offset;
	char		*name;		/* FIXME: make this a language index array... */
	void 		*dflt;
	enum pv_scale	scale;
	enum pv_unit	unit;
	void		*data;
	size_t		size;
} pv_t; 


#endif	/* !PV_TYPES_H */
